'use strict';

import {AxiosError, AxiosResponse} from 'axios';

import {Channel, Client, GuildChannel, TextChannel} from 'discord.js';
import {log, reportError} from '../core/utils';

const transferPage = require('./pages-transferer');
const axios = require('axios');
const redis = require('redis').createClient();

const vega = new Client();
const token = process.env.DISCORD_BOT_TOKEN;

const PAGE_NAME_REGEXP = /^([A-Za-z_0-9-]+)$/;

const COMMAND_REGEXP = /^[/!](\w+)/;
const DEV_CHANNEL_ID = '402913158127288321';
const STAFF_CHANNEL_ID = '363739114895441921';
const MEMBERS_ROLE_ID = '106465861476319232';

const GOOGLE_SEARCH_ENDPOINT = 'https://www.googleapis.com/customsearch/v1';
const GOOGLE_SEARCH_ID = '015370027863181820724:1ykhdit5tec';

interface GoogleSearchResults {
    data: {
        items: GoogleSearchResultItem[];
    };
}

interface GoogleSearchResultItem {
    title: string;
    link: string;
}

function search(text: string): Promise<object> {
    return axios.get(GOOGLE_SEARCH_ENDPOINT, {
        params: {
            cx: GOOGLE_SEARCH_ID,
            key: process.env.GOOGLE_API_KEY,
            q: text
        }
    })
        .catch((error: AxiosError) => {
            const response = error && error.response;
            reportError(error, {
                message: 'Search failed',
                extra: {
                    status: response && response.status,
                    headers: response && response.headers,
                    body: response && response.data
                }
            });
        })
        .then((response: AxiosResponse) => {
            const results = response.data.items
                .filter((item: GoogleSearchResultItem) => {
                    const servicePage = (
                        item.link.includes('/search:site/') ||
                        item.link.includes('most-recently-created') ||
                        item.link.includes('most-recently-edited') ||
                        item.link.includes('forum:recent-posts') ||
                        item.link.includes('draft:') ||
                        item.link.includes('top-rated-pages')
                    );
                    return !servicePage;
                })
                .slice(0, 5);

            let content = '';
            results.forEach((item: GoogleSearchResultItem) => {
                let title = item.title
                    .replace(' - The ...', '')
                    .replace(' - The SCP ...', '')
                    .replace(' - The SCP Foundation', '')
                    .replace('The SCP Foundation: ', '');

                if (item.link.includes('/forum/')) {
                    title = `Форум: ${title}`;
                }

                content += `- [${title}](${item.link})\n`;
            });

            const embed = {
                color: 16098851,
                title: 'Результаты поиска',
                description: content
            };

            return {embed};
        })
        .catch((error: Error) => {
            reportError(error, {
                message: 'Unhandled search error'
            });
        });
}

let logChannel: TextChannel;
vega.on('ready', () => {
    log('VEGA is ready');
    logChannel = vega.channels
        .find('id', process.env.DISCORD_LOG_CHANNEL_ID) as TextChannel;
});
redis.on('message', (channel: any, message: any) => {
    if (channel === 'discordLog' && logChannel) {
        logChannel.send(message);
    }
});
redis.subscribe('discordLog');

vega.on('message', (message) => {
    const match = COMMAND_REGEXP.exec(message.content);
    const command = match && match[1];
    const content = message.content.split(' ').slice(1).join(' ');

    switch (command) {
        default:
            break;
        case 'ping':
            message.channel.send('pong');
            break;
        case 'beep':
            message.channel.send('boop');
            break;
        case 'transfer': // eslint-disable-line no-case-declarations
            const allowedChannel = [DEV_CHANNEL_ID, STAFF_CHANNEL_ID].includes(message.channel.id);
            const pageNameMatch = PAGE_NAME_REGEXP.exec(content);
            const pageName = pageNameMatch && pageNameMatch[1];
            if (allowedChannel && pageName) {
                transferPage(pageName.toLowerCase(), message);
            }
            break;
        case 's':
            if (message.member.roles.has(MEMBERS_ROLE_ID)) {
                search(content).then(result => {
                    message.channel.send(result);
                })
                    .catch(e => {
                        reportError(e, {message: 'Discord message send failed'});
                    });
            }
    }
});

vega.on('error', (error) => {
    reportError(error, {
        message: 'Discord error event'
    });
    reportError(error);
});

vega.login(token).then();
